//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// layers.h -                                                                 //
//                                                                          //
// Low-level functions' prototypes                                          //
//                                                                          //
// Copyright (c) 1999 RC Module Inc.                                        //
//                                                                          //
// $Revision:: 1     $      $Date:: 26.03.99 16:34   $                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////


#include <stddef.h>


extern "C" {

    ////////////
    // SumLayer
    //  - evaluates weighted sums of a neuron layer
    // with 64-bit inputs and outputs
    // and 16-bit packed weights. Sums are saturated
    // to the [-32768..32767] range
    void SumLayer (
            size_t nInput,      // number of inputs
            size_t nNeurons,    // number of outputs (neurons)
            const long *input,
                // pointer to inputs array zero-padded to multiple
                // of 4 64-bit words
            const unsigned long *weights,
                // pointer to weights matrix, each row of packed 4x16-bit
                // values, row tails are zero-padded to the 64-bit word boundary
            long *output
                // pointer to outputs array zero-padded to multiple
                // of 32 64-bit words
        );

    //////////////
    // SumLayer12
    //  - evaluates weighted sums of a neuron layer of 12 neurons
    // with 64-bit inputs and outputs
    // and 16-bit packed weights. Sums are saturated
    // to the [-32768..32767] range
    void SumLayer12 (
            size_t nInput,      // number of inputs
            const long *input,
                // pointer to inputs array zero-padded to multiple
                // of 4 64-bit words
            const unsigned long *weights,
                // pointer to weights matrix, each row of packed 4x16-bit
                // values, row tails are zero-padded to the 64-bit word boundary
            long *output
                // pointer to outputs array of 12 64-bit words
        );

    /////////////////
    // ApplyNeuroFunc
    //  - applies neuron function (the sigmoid from -32767 to 32767)
    // to an array of 64-bit words with values in the [-32768..32767] range
    void ApplyNeuroFunc (
            size_t nInput,      // number of values
            const long *input,  // input array pointer
            long *output        // output array pointer
                                //      (may be the same as input)
        );

}   // extern "C"
